/*
 ***** BEGIN LICENSE BLOCK *****
  -   Version: MPL 1.1/GPL 2.0/LGPL 2.1
  -
  - The contents of this file are subject to the Mozilla Public License Version
  - 1.1 (the "License"); you may not use this file except in compliance with
  - the License. You may obtain a copy of the License at
  - http://www.mozilla.org/MPL/
  - 
  - Software distributed under the License is distributed on an "AS IS" basis,
  - WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
  - for the specific language governing rights and limitations under the
  - License.
  -
  - The Original Code is Strangepoint.
  -
  - The Initial Developer of the Original Code is
  - E.P. Mancini.
  - Portions created by the Initial Developer are Copyright (C) 2006
  - the Initial Developer. All Rights Reserved.
  -
  - Contributor(s):
  -
  - Bug hunters
  -  Goofy (extenzilla)
  -  Ken Arway
  -
  - Alternatively, the contents of this file may be used under the terms of
  - either the GNU General Public License Version 2 or later (the "GPL"), or
  - the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  - in which case the provisions of the GPL or the LGPL are applicable instead
  - of those above. If you wish to allow use of your version of this file only
  - under the terms of either the GPL or the LGPL, and not to allow others to
  - use your version of this file under the terms of the MPL, indicate your
  - decision by deleting the provisions above and replace them with the notice
  - and other provisions required by the GPL or the LGPL. If you do not delete
  - the provisions above, a recipient may use your version of this file under
  - the terms of any one of the MPL, the GPL or the LGPL.
  - 
  - ***** END LICENSE BLOCK ***** 

  $Id: board.js,v 1.9 2006/09/02 07:36:15 epmancini Exp $
 */
var whiteEmail="white@";
var whiteName="white";
var blackName="black";
var blackEmail="black@";

var gameName="mailchess";
var numMoves = 1;
var startDate = "09/09/1999";
var currentColor = "w";

var msgLabel

var selectedPlaceholder;

var boardImg;

// Constants for metadata
var MT_DATASEC = "[GAME DATA]";
var MT_MOVESSEC = "<moves>";
var MT_MOVESSECEND = "</moves>";
var MT_MOVESTART = "[";
var MT_MOVEEND = "]";

var MT_NAME = "name: ";
var MT_DATE = "date: ";
var MT_CURRENT = "current: ";
var MT_WHITE = "white: ";
var MT_WEMAIL = "wemail: ";
var MT_BLACK = "black: ";
var MT_BEMAIL = "bemail: ";

var MT_NUMMOV = "num: ";


var MTM_NUM = "n:";
var MTM_MOV = "m:";
var MTM_COM = "c:";
var MTM_DAT = "d:";
var MTM_FEN = "f:";
/*
 * Localization strings.
 */
var rcstrBundle;

var BOARD_BORDER = 20;
var CELL_WIDTH = 50;


/*
 * The board. It is a bidimensional array of size 8x8.
 */
var chessboard = new Array (
  new Array( 8 ),
  new Array( 8 ),
  new Array( 8 ),
  new Array( 8 ),
  new Array( 8 ),
  new Array( 8 ),
  new Array( 8 ),
  new Array( 8 )
);



// -----------------------------------------------------
// Moves management
// -----------------------------------------------------
/*
 * The list of moves.
 */
var moves = new Array();

/*
 * Make a new move.
 */
function CreateMove(m_num, m_mv, m_comment, m_date, m_fen) {
  this.num  = m_num;
  this.mv = m_mv;
  this.comment = m_comment;
  this.date = m_date;
  this.fen = m_fen;
};


/*
 * Get a string representation of a move.
 */
function MoveToString( m ) {
  return m.num +") "+m.mv+", "+m.comment+", "+m.date+". [>"+m.fen+"<]";
}

/*
 * Add a new move to the list.
 */
function AddMove(m_num, m_mv, m_comment, m_date, m_fen) {
        var m = new CreateMove(m_num, m_mv, m_comment, m_date, m_fen);
        moves.push(m);
        document.getElementById("listmoves").appendItem(m_mv, (moves.length-1));

}

// -----------------------------------------------------
// Pieces management
// -----------------------------------------------------
/*
 * All pieces with the id and the symbol. The id must 
 * be equal to the image id in board.xul.
 */
var pieces;

function initPieces() {
  pieces = new Array (
  new CreatePiece( "p_rb0", rcstrBundle.getString("r")),
  new CreatePiece( "p_nb0", rcstrBundle.getString("n")),
  new CreatePiece( "p_bb0", rcstrBundle.getString("b")),
  new CreatePiece( "p_kb",  rcstrBundle.getString("k")),
  new CreatePiece( "p_qb",  rcstrBundle.getString("q")),
  new CreatePiece( "p_bb1", rcstrBundle.getString("b")),
  new CreatePiece( "p_nb1", rcstrBundle.getString("n")),
  new CreatePiece( "p_rb1", rcstrBundle.getString("r")),
  new CreatePiece( "p_pb0", rcstrBundle.getString("p")),
  new CreatePiece( "p_pb1", rcstrBundle.getString("p")),
  new CreatePiece( "p_pb2", rcstrBundle.getString("p")),
  new CreatePiece( "p_pb3", rcstrBundle.getString("p")),
  new CreatePiece( "p_pb4", rcstrBundle.getString("p")),
  new CreatePiece( "p_pb5", rcstrBundle.getString("p")),
  new CreatePiece( "p_pb6", rcstrBundle.getString("p")),
  new CreatePiece( "p_pb7", rcstrBundle.getString("p")),

  new CreatePiece( "p_qb1", rcstrBundle.getString("q")),
  new CreatePiece( "p_qb2", rcstrBundle.getString("q")),
  new CreatePiece( "p_qb3", rcstrBundle.getString("q")),
  new CreatePiece( "p_qb4", rcstrBundle.getString("q")),
  new CreatePiece( "p_qb5", rcstrBundle.getString("q")),
  new CreatePiece( "p_qb6", rcstrBundle.getString("q")),
  new CreatePiece( "p_qb7", rcstrBundle.getString("q")),
  new CreatePiece( "p_qb8", rcstrBundle.getString("q")),
  
  new CreatePiece( "p_pw0", rcstrBundle.getString("P")),
  new CreatePiece( "p_pw1", rcstrBundle.getString("P")),
  new CreatePiece( "p_pw2", rcstrBundle.getString("P")),
  new CreatePiece( "p_pw3", rcstrBundle.getString("P")),
  new CreatePiece( "p_pw4", rcstrBundle.getString("P")),
  new CreatePiece( "p_pw5", rcstrBundle.getString("P")),
  new CreatePiece( "p_pw6", rcstrBundle.getString("P")),
  new CreatePiece( "p_pw7", rcstrBundle.getString("P")),
  new CreatePiece( "p_rw0", rcstrBundle.getString("R")),
  new CreatePiece( "p_nw0", rcstrBundle.getString("N")),
  new CreatePiece( "p_bw0", rcstrBundle.getString("B")),
  new CreatePiece( "p_kw",  rcstrBundle.getString("K")),
  new CreatePiece( "p_qw",  rcstrBundle.getString("Q")),
  new CreatePiece( "p_bw1", rcstrBundle.getString("B")),
  new CreatePiece( "p_nw1", rcstrBundle.getString("N")),
  new CreatePiece( "p_rw1", rcstrBundle.getString("R")),

  new CreatePiece( "p_qw1", rcstrBundle.getString("Q")),
  new CreatePiece( "p_qw2", rcstrBundle.getString("Q")),
  new CreatePiece( "p_qw3", rcstrBundle.getString("Q")),
  new CreatePiece( "p_qw4", rcstrBundle.getString("Q")),
  new CreatePiece( "p_qw5", rcstrBundle.getString("Q")),
  new CreatePiece( "p_qw6", rcstrBundle.getString("Q")),
  new CreatePiece( "p_qw7", rcstrBundle.getString("Q")),
  new CreatePiece( "p_qw8", rcstrBundle.getString("Q"))
);
}

/*
 * Make a new piece.
 */
function CreatePiece(nid, s) {
  this.pid  = nid;
  this.symb = s;
  this.flag = false;
};


/*
 * Set all flag to false.
 */
function ClearPieceFlags() {
  for (var i = 0; i<pieces.length; i++) {
        pieces[i].flag = false;
  }
}

/*
 * Get the piece with the specified id.
 */
function GetPieceById(p_id) {
        var p=null;
        var i=0;
        while ((p==null) && (i<pieces.length)) {
                if (pieces[i].pid.equals(p_id)) {
                        p = pieces[i];
                };
                i++;
        }
        return p;
}

/*
 * Return a piece with the choosen symbol and tje flag attribute
 * set to false. It sets the flag attribute to true.
 */
function GetPieceBySymbol(p_sym) {
        var p=null;
        var i=0;
        while ((p==null) && (i<pieces.length)) {
                if ((pieces[i].symb == p_sym) && (!pieces[i].flag)){
                        pieces[i].flag = true;
                        p = pieces[i];
                };
                
                i++;
        }
        return p;
}

/*
 *
 */
function getCellNotation(row, col) {
  var c = String.fromCharCode(104-col);
  var r= row+1;
  return c+r;
}

// -----------------------------------------------------
// Board management
// -----------------------------------------------------

/*
 * Make a string with the FEN description of the board.
 */
function Board2FEN() {
  var s = "";
  for (var r = 0; r<8; r++) {
    var c = 0;
    var voidHouses=0;
    while ( c<8) {
       if (chessboard[r][c]) {
         if (voidHouses>0) {
           s = s+voidHouses;
           voidHouses = 0;
         }
         s = s+chessboard[r][c].symb;
       } else {
         voidHouses++;
       }
       c++;
    };
    if (voidHouses>0) {
      s = s+voidHouses;
    }
    if (r<7) {
      s = s + "/";
    }
    
  };
  s = s + " "+currentColor+ " KQkq - 0 "+numMoves;
  //alert( rcstrBundle.getString( "error.funcnotimplemented")+" 'Board2FEN()'" );
  return s;
}


/*
 * Update the graphic board using the chessboard data structure.
 */
function BoardToScreen() {
   for (var i = 0; i<pieces.length; i++) {
        document.getElementById(pieces[i].pid).hidden = true;
        var menuId = "m"+pieces[i].pid.substr(1);
        document.getElementById(menuId).setAttribute('checked', 'false');
   }
   
   for (var r=0; r<8; r++) {
        for (var c=0; c<8; c++) {
                var cell=chessboard[r][c];
                if (cell!=null) {
                        var p = document.getElementById(cell.pid);   
                        var x = c*CELL_WIDTH + BOARD_BORDER;
                        var y = r*CELL_WIDTH + BOARD_BORDER;
                        p.hidden = false;
                        p.left = x;
                        p.top = y;
                        var menuId = "m"+cell.pid.substr(1);
                        document.getElementById(menuId).setAttribute('checked', 'true');
                }
        }
   }
}

/*
 * A string representation of the board.
 */
function BoardToString() {
   var status = "       abcdefgh\n      ----------\n";
   for (var r=0; r<8; r++) {
        //var rletter = String.fromCharCode(104-r)
        var rletter = r+1;
        status = status+"    "+rletter+"| ";
        for (var c=0; c<8; c++) {
                var cell=chessboard[r][c];
                if (cell!=null) {
                        status = status+cell.symb;
                        
                } else {
                        status = status+" ";
                }
        }
        status = status+" |\n";
   }
   status = status+"      ----------\n";
   return status;
}

/*
 * Set the board using a FEN string schema.
 */
function FEN2Board(s) {
    ClearPieceFlags();
    var pos = 0;
    for (var r=0; r<8; r++) {
      var c=0;
      while ((c<8) && (pos<s.length)) {
        var p_symbol = s.charAt(pos);
        var p_cod = p_symbol.charCodeAt(0) - "0".charCodeAt(0);
        if ((p_cod>0) && (p_cod<9)) {
          for (var i=0; i<p_cod; i++) {
            chessboard[r][c] = null;
            c++;
          }
        } else {
          var p = GetPieceBySymbol(p_symbol);
          if (p==null) {
            alert( "[sym:"+p_symbol+" pos:"+pos+" rc:"+r+" "+c+"] "+rcstrBundle.getString("error.fenstring")+" '"+s+"'" +p );
          }
          chessboard[r][c] = p;
          c++;
        }
        pos++;
      };
      pos++;
    }
    BoardToScreen();
}

// -----------------------------------------------------
// Game management
// -----------------------------------------------------

function ResetTo(fen, msg) {
   FEN2Board(fen);
   AddMove(numMoves, "reset", msg, dt.toLocaleString(), fen);
}


/*
 * Init the board with the opening configuration using a standard 
 * FEN string.
 */
function OpenGame() {
    var acctManager  = Components.classes["@mozilla.org/messenger/account-manager;1"].createInstance(Components.interfaces.nsIMsgAccountManager);  
    whiteName = acctManager.defaultAccount.defaultIdentity.fullName;
    whiteEmail = acctManager.defaultAccount.defaultIdentity.email;

    // Game dialog with general data
    var params = {
        wem: whiteEmail,
        wna: whiteName,
        bem: blackEmail,
        bna: blackName,
        gna: gameName,
    };
    
    var dlg = window.openDialog( "chrome://mailchess/content/datadlg.xul",  
                        "datadlg", 
                        "modal=yes,centerscreen",
                        params);
    if (params.wem==null) {
      return false;
    }
    
    whiteEmail = params.wem;
    whiteName = params.wna;
    blackEmail = params.bem;
    blackName = params.bna;
    gameName = params.gna;

    numMoves = 1;
    currentColor = "w";
    var dt = new Date;
    startDate = dt.toLocaleString();
    var fen="rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";
    FEN2Board(fen);

    // Update UI    
    AddMove(numMoves, "-", "Start", startDate, fen);

    document.getElementById("wdata").value = whiteName+" ("+whiteEmail+")";
    document.getElementById("bdata").value = blackName+" ("+blackEmail+")";
    document.getElementById("movedata").value = numMoves;
    document.getElementById("gamedata").value = gameName;

    return true;
}

/*
 * Initialization.
 */
function boardInit() {
  rcstrBundle         = document.getElementById("rcstrings");
  msgLabel            = document.getElementById("msglabel");
  selectedPlaceholder = document.getElementById("sel");
  boardImg            = document.getElementById("imgboard");
  
  initPieces();

  if (window.arguments && window.arguments[0]=="newmove") {
    emailReader()
  } else {
    if (!OpenGame()) {
      window.close();
    }
  }
  sizeToContent();
};

// -----------------------------------------------------
// Mail management
// -----------------------------------------------------

/*
 * Compose the game message body.
 */
function GetResponseBody() {
        var body = 
                rcstrBundle.getString("mail.game")+" "+gameName+"\n"+
                rcstrBundle.getString("mail.moves")+" "+numMoves+"\n"+
                rcstrBundle.getString("mail.gameStart")+" "+startDate+"\n\n"+
                document.getElementById("msgText").value+"\n\n"+
                rcstrBundle.getString("mail.WelcomeMsg")+"\n\n"+
                rcstrBundle.getString("mail.ProblemMsg")+"\n\n"+
                BoardToString()+"\n\n"+
                "\n--------------------------------------------------\n\n"+
                MT_DATASEC+"\n"+
                MT_NAME+gameName+"\n"+
                MT_DATE+startDate+"\n"+
                MT_CURRENT+currentColor+"\n"+
                MT_NUMMOV+numMoves+"\n"+
                MT_WHITE+whiteName+"\n"+
                MT_WEMAIL+whiteEmail+"\n"+
                MT_BLACK+blackName+"\n"+
                MT_BEMAIL+blackEmail+"\n"+
                MT_MOVESSEC+"\n";

        var list = document.getElementById('listmoves');
        var numItems = list.childNodes.length
        for (var index=0; index<numItems; index++) {
                var m = moves[list.getItemAtIndex(index).value];
                body = body + 
                  MT_MOVESTART+"\n"+
                  MTM_NUM+m.num+"\n"+
                  MTM_MOV+m.mv+"\n"+
                  MTM_COM+m.comment+"\n"+
                  MTM_DAT+m.date+"\n"+
                  MTM_FEN+m.fen+"\n"+
                  MT_MOVEEND+"\n";
                
        }
        body = body + MT_MOVESSECEND+"\n\n"+rcstrBundle.getString("mail.FooterMsg");
        return body;
};

/*
 * Send the game by mail.
 */
function sendGame() {
  
    var acctManager  = Components.classes["@mozilla.org/messenger/account-manager;1"].createInstance(Components.interfaces.nsIMsgAccountManager);  
    var msgCompSvc   = Components.classes["@mozilla.org/messengercompose;1"].getService(Components.interfaces.nsIMsgComposeService);
    var msgCompParam = Components.classes["@mozilla.org/messengercompose/composeparams;1"].createInstance(Components.interfaces.nsIMsgComposeParams);


    msgCompParam.identity = acctManager.defaultAccount.defaultIdentity;
    msgCompParam.type     = Components.interfaces.nsIMsgCompType.New;
    msgCompParam.format   = Components.interfaces.nsIMsgCompFormat.PlainText;
    msgCompParam.originalMsgURI = "";

    //var msgCompose = msgCompSvc.InitCompose(window, msgCompParam);

    var composeFields = Components.classes['@mozilla.org/messengercompose/composefields;1'].createInstance(Components.interfaces.nsIMsgCompFields);
    
    if (currentColor == "w") {
      composeFields.to = blackEmail;
    } else {
      composeFields.to = whiteEmail;
    }

    composeFields.body = GetResponseBody();
    composeFields.subject = "[Mailchess Game] "+gameName+" "+numMoves+currentColor;
    msgCompParam.composeFields = composeFields;
    
    /*
    if (msgCompose.editor.QueryInterface(nsIPlaintextEditorMail).wrapWidth > 0) {
    try {
      msgCompose.editor.QueryInterface(nsIPlaintextEditorMail).wrapWidth = 0 
    }
    catch (e) { }
    */
    
    msgCompSvc.OpenComposeWindowWithParams(null, msgCompParam);

    return true;
}

/*
 * Send the game email without show composer window.
 */
function sendNowGame() {
  alert( rcstrBundle.getString( "error.funcnotimplemented")+" 'Board2FEN()'" );
  //window.close();
}

/*
 * True if the first characters of string s equals the string s2.
 */
function startsWith( s, s2) {
  if (s.substr(0, s2.length)==s2) 
    return true;
  else  
    return false;
}

/*
 * Read the body of current email. It uses the current mail editor
 * and assumes the email is plain text (written with the extension itself).
 * The email has three sections: user data, metadata and moves data.
 */
function emailReader() {
    var rootElem = window.opener.GetCurrentEditor().rootElement;
    var domChilds = rootElem.childNodes;
    
    var i = 0;

    var endSection;

    // User section
    endSection = false;
    while ((i<domChilds.length) && (!endSection))  {
      if (domChilds[i].nodeValue) {
        var v = domChilds[i].nodeValue.toString();
        if (v==MT_DATASEC) {
          endSection = true;
        };
      }
      i++;
    }

    // Metadata section
    endSection = false;
    while ((i<domChilds.length) && (!endSection))  {
      if (domChilds[i].nodeValue) {
        var v = domChilds[i].nodeValue.toString();
        if (startsWith( v, MT_MOVESSEC)) {
              endSection = true;
         } else if (startsWith( v, MT_NAME)) {
              gameName = v.substr(MT_NAME.length);
         } else if (startsWith( v, MT_DATE)) {
              startDate = v.substr(MT_DATE.length);
         } else if (startsWith( v,  MT_CURRENT)) {
              currentColor = v.substr(MT_CURRENT.length);
         } else if (startsWith( v, MT_WHITE)) {
              whiteName = v.substr(MT_WHITE.length);
         } else if (startsWith( v, MT_WEMAIL)) {
              whiteEmail = v.substr(MT_WEMAIL.length);
         } else if (startsWith( v, MT_BLACK)) {
              blackName = v.substr(MT_BLACK.length);
         } else if (startsWith( v, MT_BEMAIL)) {
              blackEmail = v.substr(MT_BEMAIL.length);
         } else if (startsWith( v, MT_NUMMOV)) {
              numMoves = Number(v.substr(MT_NUMMOV.length));
        };
      }
      i++;
    }

    /*
    alert( MT_NAME+gameName+"\n"+
                MT_DATE+startDate+"\n"+
                MT_CURRENT+currentColor+"\n"+
                MT_NUMMOV+numMoves+"\n"+
                MT_WHITE+whiteName+"\n"+
                MT_WEMAIL+whiteEmail+"\n"+
                MT_BLACK+blackName+"\n"+
                MT_BEMAIL+blackEmail );
    */
    
    // Change current player color
    if (currentColor=="w") {
      currentColor = "b";
    } else {
      currentColor = "w";
      numMoves = numMoves + 1;
    }

    // Section of moves
    var m_n, m_m, m_c, m_d, m_f;
    endSection = false;
    while ((i<domChilds.length) && (!endSection))  {
      if (domChilds[i] && domChilds[i].nodeValue) {
        var v = domChilds[i].nodeValue.toString();
        if (startsWith( v, MT_MOVEEND)) {
          AddMove(m_n, m_m, m_c, m_d, m_f);

        } else if (startsWith( v, MTM_NUM)) {
          m_n = v.substr( MTM_NUM.length);
        } else if (startsWith( v, MTM_MOV)) {
          m_m = v.substr( MTM_MOV.length);
        } else if (startsWith( v, MTM_COM)) {
          m_c = v.substr( MTM_COM.length);
        } else if (startsWith( v, MTM_DAT)) {
          m_d = v.substr( MTM_DAT.length);
        } else if (startsWith( v, MTM_FEN)) {
          m_f = v.substr( MTM_FEN.length);
          
        } else if (startsWith( v, MT_MOVESSECEND)) {
          endSection = true;
        };
      }

      i++;
    }

    // Last move is the valid move
    // Update the chessboard data structure
    if (m_f) {
      //alert("FEN:"+ m_f); 
      FEN2Board( m_f );
    } else {
      alert( rcstrBundle. getString( "error.notfoundlastfen") );
    }
    
    // Update the graphical board
    BoardToScreen();
}

// -----------------------------------------------------
// Mouse commands
// -----------------------------------------------------


/*
 * The selected piece.
 */
var selectedObject  = null;

/*
 * The coordinates of selected piece.
 */
var selRow;
var selCol;

/*
 * The string that describes the move (ex. Ne4).
 */
var moveString;

/*
 * Handle the mouse clicks.
 */
function mouseDown(ev){
  
  if (selectedObject == null) {
    selCol = Math.floor( (ev.clientX - boardImg.boxObject.x - BOARD_BORDER) / CELL_WIDTH ); 
    selRow = Math.floor( (ev.clientY - boardImg.boxObject.y - BOARD_BORDER) / CELL_WIDTH );
    
    if ((ev.target.id.substr(0,2)=='p_') && 
        (selCol>=0) && (selCol<8) && (selRow>=0) && (selRow<8)){
      
      selectedObject = ev.target;
      selectedPlaceholder.left   = selCol*CELL_WIDTH+BOARD_BORDER;
      selectedPlaceholder.top    = selRow*CELL_WIDTH+BOARD_BORDER;
      selectedPlaceholder.hidden = false;
    
      //alert( selectedObject+
      //      " casa:"+selRow+","+selCol+
      //      " - evX:"+ev.clientX+" evY:"+ev.clientY+
      //      " - boardImg.x:"+boardImg.boxObject.x+"  boardImg.y:"+boardImg.boxObject.y  
      //      );
    }
    
  } else if (selectedObject) {
      var col = Math.floor( (ev.clientX - boardImg.boxObject.x-BOARD_BORDER) / CELL_WIDTH );
      var row = Math.floor( (ev.clientY - boardImg.boxObject.y-BOARD_BORDER) / CELL_WIDTH );
      var selPiece = chessboard[selRow][selCol];
      
      if ((col<0) || (col>=8) || (row<0) || (row>=8)) {
        // Click out of the board
        
      } else if ((col==selCol) && (row==selRow)) {
        // Click on the same house
        selectedObject = null;
        selectedPlaceholder.hidden = true;
        
      } else if (selPiece && isValidMove(selPiece.symb, selRow, selCol, row, col)) {
        // Click on different house
        moveString = selPiece.symb+getCellNotation(selRow,selCol);
        var doMove = true;
        if (chessboard[row][col]) {
          // Captured
          if (confirm(rcstrBundle.getString( "queryCapture")+" ("+moveString+'x'+getCellNotation(row,col)+")")) {
            //document.getElementById(chessboard[row][col].pid).hidden = true;
            TogglePiece( chessboard[row][col].pid );
            moveString = moveString+'x';
          } else {
            doMove = false;
          }
        }
        
        if (doMove) {
          chessboard[selRow][selCol] = null;  
          chessboard[row][col] = selPiece;  
          moveString = moveString+
                       getCellNotation(row,col)+
                       document.getElementById("msgMoveText").value;
          
          var dt = new Date;
          AddMove( numMoves, moveString, "", dt.toLocaleString(), Board2FEN() );
          
          selectedObject = null;
          selectedPlaceholder.hidden = true;
        }
        
        BoardToScreen();
      }
    //alert( selectedObject+" evX:"+ev.clientX+" evY:"+ev.clientY );
  }
    
  return false;
}

/*
 * Check if a move is valid.
 */
function isValidMove( symbol, sourceRow, sourceCol, destRow, destCol ) {
  return true;
}

/*
 * Handle clicks on moves listbox.
 */
function ShowLBMove() {
  alert("click");
}

/*
 * Handle hiding of pieces.
 */
function TogglePiece(p_id) {
  var menuId = "m"+p_id.substr(1);
  if (p_id.hidden) {
    p_id.hidden = false;
    document.getElementById(menuId).setAttribute('checked', "true");
  } else {
    p_id.hidden = true;
    document.getElementById(menuId).setAttribute('checked', "false");
  }
}
